#include <ctype.h>
#include "hdf.h"
#include "hdfutil.h"


/* -----------------------------------------------------------------
 NAME
       hdf_print
 DESCRIPTION
       Print a group of HDF data of same type into an ASCII file.
       The HDF data are inputted in the value buffer. The seperation of
       values in the ASCII file is a space. A newline is printed at the  
       end of group.
 RETURNS
       None
 AUTHOR
       Dr. Liping Di, Hughes STX.  
 COMMENTS
          
 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.

---------------------------------------------------------------------*/

void hdf_print(FILE *fp,         /* In: ASCII file pointer */ 
              int32 num_type,   /* In: The HDF type of values to be printed */ 
              int count,        /* In: The number of values in the value buffer */
              void *value)      /* In: the value buffer */
{   int32 step, i, number;
    char *type;
    double buffer[128];

    number = 0;
    switch(num_type){
    case DFNT_FLOAT64:
          step = 8;
          memcpy(buffer, value, 8*count);
          break;
    case DFNT_FLOAT32:
    case DFNT_INT32:
    case DFNT_UINT32:
          step = 4;
          memcpy(buffer, value, 4*count);
          break;
    case DFNT_INT16:
    case DFNT_UINT16:
          step=2;
          memcpy(buffer, value, 2*count);
          break;
    case DFNT_UINT8:
    case DFNT_INT8:
          step = 1;
          break;
    case DFNT_UCHAR8:
    case DFNT_CHAR8:
          step = 1;
          break;
    default:
          step = 0;
    }
    type = (char *)value;
    if(step != 1)type = (char *)buffer;
    for(i=0; i < count; i++){
          switch (num_type) {
          case 5:     /* float32 type. */
               fprintf(fp,"%13E",*(float32 *)type);
               break;
          case 6:     /* float64 type. */
               fprintf(fp,"%13E", *(float64 *)type);
               break;
          case 20:/* int8 type. */
               fprintf(fp,"%4d", *(int8 *)type);
               break;
          case 21:/* uint8 type. */
               fprintf(fp,"%3u", *(uint8 *)type);
               break;
          case 22:/* int16 type. */
               fprintf(fp,"%6d", *(int16 *)type);
               break;
          case 23:/* uint16 type. */
               fprintf(fp,"%5u", *(uint16 *)type);
               break;
          case 24:/* int32 type. */
               fprintf(fp,"%11d", *(int32 *)type);
               break;
          case 25:/* uint32 type. */
               fprintf(fp,"%10u", *(uint32 *)type);
               break;
          case DFNT_CHAR8:
          case DFNT_UCHAR8:
               if(isgraph(*(char8 *)type))
                    fprintf(fp,"%c", *(char8 *)type);
               else
                    fprintf(fp," ");
               break;
          default: 
               fprintf(fp,"Error: Unknown type");
          }

          if(i != count-1){
               if(num_type != DFNT_CHAR8 && num_type != DFNT_UCHAR8)fprintf(fp," ");
               type = type + step;               
          }
     }
}


/*--------------------------------------------------------------------
 NAME
       hdf_minmax
 DESCRIPTION
       find the minimum and maximum values from a field from a group of HDF 
       vdata records stored in a buffer. Assuming the field interested starts
       at buffer[0] (Note: in order to do that, the calling routine only needs 
       to pass the address of the start position of the field interested). 
   
 RETURNS
       -1: failed, 0 successful.
 AUTHOR
       Dr. Liping Di, Hughes STX.  
 COMMENTS
   
 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.
---------------------------------------------------------------------*/

int hdf_minmax(void *buffer,      /* In:  Buffer containing vdata records,
                                          buffer[0] is the start position 
                                          of the field interested */
           int16 nt,              /* In:  The data type for the field */
           int n_elements,        /* In:  Number of (elements) records in 
                                          the buffer */
           int nbytes,            /* In:  number of bytes per vdata records */
           double *vmin,          /* Out: The minimum value of the data */
           double *vmax)          /* Out: The maximum value of the data */
{
unsigned char *pbuf, u8max, u8min;

char *ptr_char, c8max, c8min;
int16 i16, i16max, i16min;
uint16 u16, u16max, u16min;
int32 i32, i32max, i32min;
uint32 u32, u32max, u32min;
float32 f32, f32max, f32min;
float64 f64, f64max, f64min;

int i, j;
  
  ptr_char = (char *)buffer;

  switch(nt){
  
  case DFNT_CHAR8:
  case DFNT_INT8:
       
            c8max = c8min = *ptr_char;
            for(i=0; i< n_elements; i++){
                 if(*ptr_char > c8max)c8max = *ptr_char;
                 if(*ptr_char < c8min)c8min = *ptr_char;
                 ptr_char += nbytes;
            } 
                 
            *vmax = c8max;
            *vmin = c8min;

            break;     

  case DFNT_UCHAR8:
  case DFNT_UINT8:
            pbuf = (unsigned char *)ptr_char;
            u8max = u8min = *pbuf;
            for(i=0; i<n_elements; i++){
                  if(*pbuf > u8max)u8max = *pbuf;
                  if(*pbuf < u8min)u8min = *pbuf;
                  pbuf += nbytes;
            }
            *vmin = u8min;
            *vmax = u8max;
            break;     

  case DFNT_UINT16:
  
            memcpy(&u16min, ptr_char, sizeof(uint16));
            u16max = u16min;
            for(i=0; i< n_elements; i++){
                 memcpy(&u16, ptr_char, sizeof(uint16));
                 if(u16 > u16max)u16max = u16;
                 if(u16 < u16min)u16min = u16;
                 ptr_char += nbytes;
            }
            *vmin = u16min;
            *vmax = u16max;
            break;     
                 
  case DFNT_INT16:
  
            memcpy(&i16min, ptr_char, sizeof(int16));
            i16max = i16min;
            for(i=0; i < n_elements; i++){
                 memcpy(&i16, ptr_char, sizeof(int16));
                 if(i16 > i16max)i16max = i16;
                 if(i16 < i16min)i16min = i16;
                 ptr_char += nbytes;
            }
            *vmin = i16min;
            *vmax = i16max;
            break;     

  case DFNT_UINT32:
  
            memcpy(&u32min, ptr_char, sizeof(uint32));
            u32max = u32min;
            for(i=0; i < n_elements; i++){
                 memcpy(&u32, ptr_char, sizeof(uint32));
                 if(u32 > u32max)u32max = u32;
                 if(u32 < u32min)u32min = u32;
                 ptr_char += nbytes;
            }

            *vmin = u32min;
            *vmax = u32max;
            break;     

  case DFNT_INT32:
  
            memcpy(&i32min, ptr_char, sizeof(int32));
            i32max = i32min;
            for(i=0; i< n_elements; i++){
                 memcpy(&i32, ptr_char, sizeof(int32));
                 if(i32 > i32max)i32max = i32;
                 if(i32 < i32min)i32min = i32;
                 ptr_char += nbytes;
            }

            *vmin = i32min;
            *vmax = i32max;
            break;
          
  case DFNT_FLOAT32:
  
            memcpy(&f32min, ptr_char, sizeof(float32));
            f32max = f32min;
            for(i=0; i < n_elements; i++){
                 memcpy(&f32, ptr_char, sizeof(float32));
                 if(f32 > f32max)f32max = f32;
                 if(f32 < f32min)f32min = f32;
                 ptr_char += nbytes;
            }
            *vmin = f32min;
            *vmax = f32max;
            break;

  case DFNT_FLOAT64:
  
            memcpy(&f64min, ptr_char, sizeof(float64));
            f64max = f64min;
            for(i=0; i < n_elements; i++){
                 memcpy(&f64, ptr_char, sizeof(float64));
                 if(f64 > f64max)f64max = f64;
                 if(f64 < f64min)f64min = f64;
                 ptr_char += nbytes;
            }
            *vmin = f64min;
            *vmax = f64max;
            break;
            
       default:
            return -1;
 }
 return 0;
}

/*--------------------------------------------------------------------
 NAME
       hdf_scaling
 DESCRIPTION
       Linearly scaling a field in a group of HDF vdata records to a specific 
       data range starting from 0 based on the minimum and maximum value of
       the field. The scaled data is stored in an unsingned short buffer supplied
       by the calling routine. 

 RETURNS
       -1: failed, 0 successful.
 AUTHOR
       Dr. Liping Di, Hughes STX.  
 COMMENTS
   
 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.
---------------------------------------------------------------------*/

int hdf_scaling(void *buffer,     /* In:  Buffer containing vdata records,
                                          buffer[0] is the start position 
                                          of the field interested */
                int nt,           /* In:  The HDF data type for the field */
                int n_elements,   /* In:  Number of elements (records) in buffer */
                int nbytes,       /* In:  Number of bytes for one record */
                int n_levels,     /* In:  Number of data levels after scaling.
                                          The data will be scaled to the range
                                          from 0 to n_levels-1. */
                double vmin,      /* In:  The minimum value of the field */
                double vmax,      /* In:  The maximum value of the field */
                unsigned short *out)  /* Out: The buffer containing the scaled
                                              value. The buffer should be
                                              supplied by the calling routine
                                              and at least n_elements long */
{
unsigned short look_up[256];
unsigned char *pbuf, u8max, u8min;

char *ptr_char, c8max, c8min;
int16 i16, i16max, i16min;
uint16 u16, u16max, u16min;
int32 i32, i32max, i32min;
uint32 u32, u32max, u32min;
float32 f32, f32max, f32min, a1, b1;
float64 f64, f64max, f64min;
float64 a, b;

int i, j;

  if (n_levels < 1 || n_levels > 65535) return -1;

  if (vmax == vmin){
      a1 = a = 0;
      b1 = b = n_levels/2;
  }
  else {  
       a1 = a = n_levels/(vmax-vmin);
       b1 = b = -a*vmin;
  }

  ptr_char = (char *)buffer;
  switch(nt){
  
  case DFNT_CHAR8:
  case DFNT_INT8:
            for(i=0; i < 256; i++)
                 look_up[i] = (unsigned short )(a1 * (i - 128)+ b1 + 0.1);
           
            for(i = 0; i< n_elements; i++){
                 out[i] = look_up[*ptr_char + 128];
                 ptr_char += nbytes;
            }
                 
            break;     

  case DFNT_UCHAR8:
  case DFNT_UINT8:
            for(i = 0; i < 256; i++)
                 look_up[i] = (unsigned short)(a1 * i + b1 + 0.1);

            pbuf = (unsigned char *)buffer;
            for(i = 0; i< n_elements; i++){
                  out[i] = look_up[*pbuf];
                  pbuf += nbytes;
            }

            break;     

  case DFNT_UINT16:
            for(i = 0; i< n_elements; i++){
                  memcpy(&u16, ptr_char, sizeof(uint16));
                  out[i] = (unsigned short)(u16 * a1 + b1);
                  ptr_char += nbytes;
            }
            break;     
                 
  case DFNT_INT16:
            for(i = 0; i< n_elements; i++){
                 memcpy(&i16, ptr_char, sizeof(int16));
                 out[i] = (unsigned short)(i16*a1 + b1);
                 ptr_char += nbytes;
            }
            break;     

  case DFNT_UINT32:
            for(i = 0; i< n_elements; i++){
                 memcpy(&u32, ptr_char, sizeof(uint32));
                 out[i] = (unsigned short)(u32*a1 + b1);
                 ptr_char += nbytes;
            }
            break;     

  case DFNT_INT32:
            for(i = 0; i< n_elements; i++){
                 memcpy(&i32, ptr_char, sizeof(int32));
                 out[i] = (unsigned short)(i32*a1 + b1);
                 ptr_char += nbytes;
            }
            break;
          
  case DFNT_FLOAT32:
            for(i = 0; i< n_elements; i++){
                 memcpy(&f32, ptr_char, sizeof(float32));
                 out[i] = (unsigned short)(f32*a1 + b1);
                 ptr_char += nbytes;
            }
            break;

  case DFNT_FLOAT64:
            for(i = 0; i< n_elements; i++){
                 memcpy(&f64, ptr_char, sizeof(float64));
                 out[i] = (unsigned short)(f64*a + b);
                 ptr_char += nbytes;
            }
            break;
            
       default:
            return -1;
 }
 return 0;
}

/*--------------------------------------------------------------------
 NAME
       hdf_mmscaling
 DESCRIPTION
       This routine first calculates the minimum and maximum values of 
       a group of HDF data with same data type, then linearly scale the 
       field to a specific data range starting from 0 based on the 
       minimum and maximum value of the field. The scaled data is stored 
       in an unsingned short buffer supplied by the calling routine. 

 RETURNS
       -1: failed, 0 successful.
 AUTHOR
       Dr. Liping Di, Hughes STX.  
 COMMENTS
   
 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.
---------------------------------------------------------------------*/

int hdf_mmscaling(void *buffer,   /* In:  Buffer containing HDF data with
                                          same data type */ 
                int nt,           /* In:  The HDF data type */
                int n_elements,   /* In:  Number of data elements in buffer */
                int n_levels,     /* In:  Number of data levels after scaling.
                                          The data will be scaled to the range
                                          from 0 to n_levels-1. */
                unsigned short *out,  /* Out: The buffer containing the scaled
                                              value. The buffer should be
                                              supplied by the calling routine
                                              and at least n_elements long */
                double *vmin,         /* Out: The minimum value calculated */
                double *vmax)         /* Out: The maximum value calculated */
{
unsigned short look_up[256];
unsigned char *pbuf, u8max, u8min;

char *ptr_char, c8max, c8min;
int16 *ptr_i16, i16max, i16min;
uint16 *ptr_u16, u16max, u16min;
int32 *ptr_i32, i32max, i32min;
uint32 *ptr_u32, u32max, u32min;
float32 *ptr_f32, f32max, f32min, a1, b1;
float64 *ptr_f64, f64max, f64min;
float64 a, b;
int16 *p2buf;

int i;
  
  pbuf = (unsigned char *)buffer;
  p2buf = (int16 *)buffer;

  switch(nt){
  
  case DFNT_CHAR8:
  case DFNT_INT8:
       
            ptr_char = (char *)buffer;
           
            c8max = c8min = *ptr_char;
            for(; ptr_char < (char *)buffer + n_elements; ptr_char++){
                 if(*ptr_char > c8max)c8max = *ptr_char;
                 if(*ptr_char < c8min)c8min = *ptr_char;
            }
            if(c8max == c8min)
                 look_up[0]= n_levels/2;
            else {
                 a =(float32)n_levels/(c8max-c8min);
                 b = -a*c8min;
                 
                 for(i=c8min; i<= c8max; i++)
                       look_up[i-c8min] = (unsigned short)(a*i+b+0.1);
            }
            ptr_char = (char *)buffer;
            for(i = 0; i< n_elements; i++){
                 out[i] = look_up[*ptr_char - c8min];
                 ptr_char ++;
            }
                 
            *vmax = c8max;
            *vmin = c8min;
            break;     

  case DFNT_UCHAR8:
  case DFNT_UINT8:
            u8max = u8min = *pbuf;
            for(; pbuf < (unsigned char *)buffer + n_elements; pbuf++){
                  if(*pbuf > u8max)u8max = *pbuf;
                  if(*pbuf < u8min)u8min = *pbuf;
            }
            if(u8max == u8min)
                  look_up[0]=n_levels/2;
            else {
                  a =(float64)n_levels/(u8max - u8min);
                  b = -a*u8min;
                 
                  for(i=u8min; i<= u8max; i++)
                       look_up[i-u8min] = (unsigned short)(a*i+b+0.1);
            }

            pbuf = (unsigned char *)buffer;
            for(i = 0; i< n_elements; i++){
                  out[i] = look_up[*pbuf - u8min];
                  pbuf ++;
            }

            *vmin = u8min;
            *vmax = u8max;
            break;     

  case DFNT_UINT16:
  
            ptr_u16 = (uint16 *)buffer;
            u16max = u16min = *ptr_u16;
            for(; ptr_u16 < (uint16 *)buffer + n_elements; ptr_u16++){
                 if(*ptr_u16 > u16max)u16max = *ptr_u16;
                 if(*ptr_u16 < u16min)u16min = *ptr_u16;
            }
            if(u16max == u16min){
                 for(i=0; i< n_elements; i++)
                      out[i]=n_levels/2;
            }
            else {
                 a1 = (float32)n_levels/(u16max - u16min);
                 b1 = (-a1*u16min) + 0.5;
            
                 ptr_u16 = (uint16 *)buffer;                 
                 for(i = 0; i< n_elements; i++){
                      out[i] = (unsigned short)(*ptr_u16*a1 + b1);
                      ptr_u16++;
                 }
            }
            *vmin = u16min;
            *vmax = u16max;
            break;     
                 
  case DFNT_INT16:
  
            ptr_i16 = (int16 *)buffer;
            i16max = i16min = *ptr_i16;
            for(; ptr_i16 < (int16 *)buffer + n_elements; ptr_i16++){
                 if(*ptr_i16 > i16max)i16max = *ptr_i16;
                 if(*ptr_i16 < i16min)i16min = *ptr_i16;
            }
            if(i16max == i16min){
                 for(i=0; i< n_elements; i++)
                      out[i] = n_levels/2;
            }
            else {
                 a1 =(float32)n_levels/(i16max - i16min);
                 b1 = (-a1*i16min) + 0.5;

                 ptr_i16 = (int16 *)buffer;
            
                 for(i = 0; i< n_elements; i++){
                      out[i] = (unsigned short)(*ptr_i16*a1 + b1);
                      ptr_i16++;
                 }
            }
            *vmin = i16min;
            *vmax = i16max;
            break;     

  case DFNT_UINT32:
  
            ptr_u32 = (uint32 *)buffer;
            u32max = u32min = *ptr_u32;
            for(; ptr_u32 < (uint32 *)buffer + n_elements; ptr_u32++){
                 if(*ptr_u32 > u32max)u32max = *ptr_u32;
                 if(*ptr_u32 < u32min)u32min = *ptr_u32;
            }
            if(u32max == u32min){
                 for(i=0; i < n_elements; i++)
                      out[i] = n_levels/2;
            }
            else {
                 a1 =(float32)n_levels/(u32max - u32min);
                 b1 = (-a1*u32min) + 0.5;

                 ptr_u32 = (uint32 *)buffer;
                 for(i = 0; i< n_elements; i++){
                      out[i] = (unsigned short)(*ptr_u32*a1 + b1);
                      ptr_u32++;
                 }
            }
            *vmin = u32min;
            *vmax = u32max;
            break;     

  case DFNT_INT32:
  
            ptr_i32 = (int32 *)buffer;
            i32max = i32min = *ptr_i32;
            for(; ptr_i32 < (int32 *)buffer + n_elements; ptr_i32++){
                 if(*ptr_i32 > i32max)i32max = *ptr_i32;
                 if(*ptr_i32 < i32min)i32min = *ptr_i32;
            }
            if(i32max == i32min){
                 for(i=0; i< n_elements; i++)
                      out[i] = (unsigned short)n_levels/2;
            }
            else {
                a1 = (float32)n_levels/(i32max - i32min);
                b1 = (-a1*i32min) + 0.5;

                ptr_i32 = (int32 *)buffer;
                for(i = 0; i< n_elements; i++){
                      out[i] = (unsigned short)(*ptr_i32*a1 + b1);
                      ptr_i32++;
                 }
            }
            *vmin = i32min;
            *vmax = i32max;
            break;
          
  case DFNT_FLOAT32:
  
            ptr_f32 = (float32 *)buffer;
            f32max = f32min = *ptr_f32;
            for(; ptr_f32 < (float32 *)buffer + n_elements; ptr_f32++){
                 if(*ptr_f32 > f32max)f32max = *ptr_f32;
                 if(*ptr_f32 < f32min)f32min = *ptr_f32;
            }
            if(f32max == f32min){
                 for(i=0; i< n_elements; i++)
                      out[i] = n_levels/2;
            }     
            else {
                 a1 = n_levels/(f32max - f32min);
                 b1 = (-a1*f32min) + 0.5;

                 ptr_f32 = (float32 *)buffer;
                 for(i = 0; i< n_elements; i++){
                      out[i] = (unsigned short)(*ptr_f32*a1 + b1);
                      ptr_f32++;
                 }
            }
            *vmin = f32min;
            *vmax = f32max;
            break;

  case DFNT_FLOAT64:
  
            ptr_f64 = (float64 *)buffer;
            f64max = f64min = *ptr_f64;
            for(; ptr_f64 < (float64 *)buffer + n_elements; ptr_f64++){
                 if(*ptr_f64 > f64max)f64max = *ptr_f64;
                 if(*ptr_f64 < f64min)f64min = *ptr_f64;
            }
            if(f64max == f64min){
                 for(i=0; i< n_elements; i++)
                      out[i] = (unsigned short)n_levels/2;
            }
            else {
                 a = n_levels/(f64max - f64min);                
                 b = (-a*f64min) + 0.5;

                 ptr_f64 = (float64 *)buffer;
                 for(i = 0; i< n_elements; i++){
                      out[i] = (unsigned short)(*ptr_f64*a + b);
                      ptr_f64++;
                 }
            }
            *vmin = f64min;
            *vmax = f64max;
            break;
            
       default:
            return -1;
 }
 return 0;
}



