/*--------------------------------------------------------------------
 NAME
       plotgif
 DESCRIPTION
       Draw a XY graph on a bit map and convert the bit map to a gif.
 RETURNS
       file name for the subseted data.
 AUTHOR
       Dr. Liping Di, Hughes STX.  
 COMMENTS
   
 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.
---------------------------------------------------------------------*/
#include <math.h>
#include "sdb.h"
#include "plotgif.h"
#include "hdfutil.h"
#include "plot.h"

extern int GetGIFPixel_my(int, int);
extern unsigned char *pixel_buffer;
extern int mywidth;
extern int GIFEncode();

int plotgif(char *ftmp,             /* input: gif file name */
              unsigned short bxsize,  /* input: bit map size in x (cols) */
              unsigned short bysize,  /* input: bit map size in y (rows) */
              void *xvalue,           /* input: x value unscaled */ 
              void *yvalue,           /* input: y value unscaled */
              int style,
              int32   xtype,        /* input: data type for x in HDF notation */
              int32   *ytype,       /* input: data type for y in HDF notation */
              int32 *loc,    /* input: the location of individual y variables */
              int  xsize,           /* the number of bytes between x records */
              int ysize,            /* the number of bytes between y records */
              int nvars,            /* number of y variables */
              int scaling,          /* for multiple Y, scaling 1= relative
                                       2 = absolute */
              long nrecords,        /* number of records */
              char *xname,          /* the x axis name */
              char *yname,          /* the y axis name */
              char *plot_title,     /* title, in case of an error, the message 
                                         will be sent back in this buffer */
              char *fields         /* comma delimited variable names, the first one is X */
){
    double vmin, vmax, xmax=0, xmin=0, ymax=0, ymin=0;
    double dif, step, minvalue, y1max=0, y1min=0;
    unsigned short *x=NULL, **y=NULL, gxsize, gysize;
    XY_PLOT plot;
    FILE *fp = NULL;
    char *pt1;
    short j, i;
    int *reds, *greens, *blues;
    int yvtype;
    char *field_names;

   if((plot.bit_map = (unsigned char *)malloc(bxsize*bysize)) == NULL){
          strcpy(plot_title, "out of memory");
          return(0);
   }
   plot.x = x = (unsigned short *)malloc(nrecords * sizeof(unsigned short));
   plot.y = y = (unsigned short **)malloc(nvars * sizeof (short **));
   for ( i = 0; i < nvars; i++)
       y[i] = (unsigned short *)malloc(nrecords * sizeof(unsigned short));
   if(x == NULL || y [nvars - 1] == NULL){
        if( x != NULL) free(x);
        for ( i= 0; i< nvars; i++)
            if(y[i] != NULL) free(y[i]);
        free(y);
        strcpy(plot_title, "out of memory");
        return(0);
   }

   plot.var_names = (char **)malloc((nvars + 1) * sizeof (char **));
   field_names = (char *)malloc( strlen(fields) + 1);
   strcpy(field_names, fields);
   DBUG_PRINT(3, (LOGF, "field_names=%s\n", field_names));
   for(i=0; i<= nvars; i++){ /* total nvars+1 variable names, the first one is x */
       if(i == 0)
          pt1 = strtok(field_names, ",");
       else
          pt1 = strtok(NULL, ",");
       plot.var_names[i] = pt1;
       DBUG_PRINT(3, (LOGF, "var_name[%d]=%s\n", i, pt1));
   }
   plot.n_pairs = (long *)malloc(nvars * sizeof (long));
   plot.plot_pattern = (short *)malloc(nvars * sizeof (short));
   plot.plot_value = (short *)malloc (nvars *sizeof (short));

   plot.xsize = bxsize;     /* bit map size in x direction= number of columns */
   plot.ysize = bysize;     /* bit map size in y direction= number of rows */
   plot.bvalue= 0;          /* the background value to be set */

   plot.nvars = nvars;
   for (i = 0; i< nvars; i++){
       plot.n_pairs[i]= nrecords;      /* number of (x, y) pairs */
       plot.plot_pattern[i]=style;     /* -1 = dot, 0= soild line ... 6 */
       plot.plot_value[i]=i+2;         /* The forground value to be plot */
   }

   plot.x_axis_start=G_LEFTMARGIN;
   plot.x_axis_end=bxsize-G_RIGHTMARGIN;   /* start and end of x axis */
   plot.y_axis_start=G_BOTTONMARGIN;
   plot.y_axis_end=bysize-G_TOPMARGIN;     /* start and end of y axis */
   plot.axis_value=1;                      /* axis value (color) */
   plot.frame=1;                           /* 0 = no frame, 1 = frame */ 
   gxsize = plot.x_axis_end - plot.x_axis_start;
   gysize = plot.y_axis_end - plot.y_axis_start;

   /* scaling the plot value */
   if( xtype != DFNT_CHAR8){
        hdf_minmax(xvalue, xtype, nrecords, xsize, &xmin, &xmax);
        dif = (xmax -xmin);
        xmax += 0.1 * dif;
        xmin -= 0.1 * dif;
        hdf_scaling(xvalue, xtype, nrecords, xsize, gxsize, xmin, xmax, x);
   }
   else {
         for(i=0; i< nrecords; i++)
              x[i]=i;
         hdf_minmax(x, DFNT_UINT16, nrecords, xsize, &xmin, &xmax);
         hdf_scaling(xvalue, DFNT_UINT16, nrecords, xsize, gxsize, xmin, xmax, x);
   }

   /* scaling the y variables */
   if ( scaling == 1 ){         /* relative scaling */
       for ( i = 0; i<nvars; i++){
            hdf_minmax((char *)yvalue + loc[i], ytype[i], nrecords, ysize, &vmin, &vmax);
            dif = vmax - vmin;
            vmax += 0.1 * dif;
            vmin -= 0.1 * dif;
            hdf_scaling((char *)yvalue + loc[i], ytype[i], nrecords, 
                          ysize, gysize, vmin, vmax, y[i]);
            DBUG_PRINT(3, (LOGF, "ymin=%f ymax=%f\n", vmin, vmax));

            /* keep first two maximum and minimum values */
            if( i == 0){
                ymin = vmin;
                ymax = vmax;
            }
            else if ( i == 1){
                y1min = vmin;
                y1max = vmax;
            }
       }
   }
   else {        /* absolute scaling, all data scales into one min & max */
        for ( i=0; i<nvars; i++){
            hdf_minmax((char *)yvalue + loc[i], ytype[i], nrecords, ysize, &vmin, &vmax);
            if(i == 0){
                 ymin = vmin;
                 ymax = vmax;
            }
            else {
                 if ( ymin > vmin ) ymin = vmin;
                 if ( ymax < vmax ) ymax = vmax;
            }
        }
        dif = (ymax - ymin) * 0.1;
        ymax += dif;
        ymin -= dif;
        for ( i = 0; i<nvars; i++)
            hdf_scaling((char *)yvalue + loc[i], ytype[i], nrecords, ysize, 
                          gysize, ymin, ymax, y[i]);
   }

   for(i=0; i< nrecords; i++)
         x[i] += G_LEFTMARGIN;
   
   for(i=0; i< nvars; i++)
        for(j=0; j < nrecords; j++)
            y[i][j] += G_BOTTONMARGIN;

   DBUG_PRINT(3,(LOGF, "xmax=%f, xmin=%f\n", xmax, xmin)); 
   DBUG_PRINT(3,(LOGF, "ymax=%f, ymin=%f\n", ymax, ymin));

   /* x axis tics and labels */
   DBUG_PRINT(3, (LOGF, "before x_tic\n"));
   plot.label_x = tic_label(xmin, xmax, gxsize,&(plot.tic_start_x), 
                            &(plot.tic_step_x), &(plot.n_major_tic_x),
                            &(plot.mtic_start_x), &(plot.n_minor_tic_x));
  
   DBUG_PRINT(3, (LOGF, "after x tic\n"));
   plot.tic_start_x += G_LEFTMARGIN;      /* start location of tic in x axis */
   plot.mtic_start_x += G_LEFTMARGIN; 
   plot.mtic_step_x = plot.tic_step_x;

   /* for character X axis, recreate the label */
   if(xtype == DFNT_CHAR8){
       for(i=0; i< plot.n_label_x; i++){
          if(plot.label_x[i] != NULL) free(plot.label_x[i]);
          plot.label_x[i] = (char *)malloc(xsize + 1);
          j = (i*plot.tic_step_x)*step+minvalue+0.5;
          j = j*xsize;
          memcpy(plot.label_x[i], ((char *)xvalue) + j, xsize);
          plot.label_x[i][xsize] = '\0';
       }
   }
   plot.label_value=1;                   /* value used for label ploting */
   /* create X label location */
   plot.n_label_x=plot.n_major_tic_x;    /* number of lables for x */
   plot.x_label_x=plot.tic_start_x;
   plot.x_label_y=G_BOTTONMARGIN - 5;    /* xy coordinates of the start label */
   plot.x_label_step=plot.tic_step_x;

   for(i=0; i < plot.n_label_x; i++){
      DBUG_PRINT(3, (LOGF, "label[%d]=%s\n", i, plot.label_x[i]));
   }   

   /* set to no labels and tics first */
   plot.n_label_y = plot.n_major_tic_y = plot.n_minor_tic_y = 0;
   plot.n_1label_y = plot.n_1major_tic_y = plot.n_1minor_tic_y = 0;
   plot.label_y = plot.label_1y = NULL;

   /* draw the first Y axis tics and labels */
   if(scaling != 1 || nvars <= 2){
       plot.label_y = tic_label(ymin, ymax, gysize,&(plot.tic_start_y), 
                                &(plot.tic_step_y), &(plot.n_major_tic_y),
                                &(plot.mtic_start_y), &(plot.n_minor_tic_y));
  
       DBUG_PRINT(3, (LOGF, "after Y tic\n"));
       plot.tic_start_y += G_BOTTONMARGIN;   /* start location of tic in x axis */
       plot.mtic_start_y += G_BOTTONMARGIN; 
       plot.mtic_step_y = plot.tic_step_y;

       /* create the y label location */
       plot.n_label_y=plot.n_major_tic_y;    /* number of lables for y */
       plot.y_label_x=G_LEFTMARGIN-5;
       plot.y_label_y= plot.tic_start_y;     /* xy coordinates of the start label */
       plot.y_label_step=plot.tic_step_y;
    }

    plot.y_1axis_name = NULL;
    /* draw scond tics and labels */
    if(nvars == 2 && scaling == 1){  /* draw the second tics and labels */
       plot.label_1y = tic_label(y1min, y1max, gysize,&(plot.tic_1start_y), 
                                &(plot.tic_1step_y), &(plot.n_1major_tic_y),
                                &(plot.mtic_1start_y), &(plot.n_1minor_tic_y));
  
       plot.tic_1start_y += G_BOTTONMARGIN;   /* start location of tic in x axis */
       plot.mtic_1start_y += G_BOTTONMARGIN; 
       plot.mtic_1step_y = plot.tic_1step_y;
    
      /* create the y1 label location */
      plot.n_1label_y=plot.n_1major_tic_y;    /* number of lables for y1 */
      plot.y_1label_x = gxsize+G_LEFTMARGIN+5;
      plot.y_1label_y= plot.tic_1start_y;     /* xy coordinates of the start label */
      plot.y_1label_step=plot.tic_1step_y;
      if(yname != NULL){
          pt1 = strchr(yname, ';');
          if(pt1){
             *pt1 = '\0';
             pt1 ++;
          }
          plot.y_1axis_name = pt1;
      }
      if(plot.y_1axis_name == NULL) 
          plot.y_1axis_name = plot.var_names[2];
      plot.yname_1start_x = G_LEFTMARGIN + gxsize + 30;
   }
   DBUG_PRINT(3, (LOGF, "yname_1=%s\n", plot.y_1axis_name));

   plot.name_value=1;                    /* color for the name */
   plot.x_axis_name=xname;               /* name of x axis */
   plot.xname_start_x=100;
   plot.xname_start_y=G_BOTTONMARGIN - 30;

   plot.y_axis_name=yname;
   plot.yname_start_x= G_LEFTMARGIN - 30;
   plot.yname_start_y= 100;

   plot.title_value=1;
   plot.title=plot_title;
   plot.title_start_x=100;
   plot.title_start_y=B_YSIZE - 25;

   DBUG_PRINT(3, (LOGF, "Before ploting\n"));

   xy_plot(&plot);

   DBUG_PRINT(3, (LOGF, "After Ploting\n"));
   for(i=0; i < plot.n_label_x; i++)
        if(plot.label_x[i] != NULL) free(plot.label_x[i]);
   free (plot.label_x);
   for(i=0; i < plot.n_label_y; i++)
        if(plot.label_y[i] != NULL)free(plot.label_y[i]);
   for(i=0; i < plot.n_1label_y; i++)
        if(plot.label_1y[i] != NULL)free(plot.label_1y[i]);
   if(plot.label_y) free(plot.label_y);
   if(plot.label_1y)free(plot.label_1y);
   free(plot.x);
   for(i=0; i< nvars; i++)
       free(plot.y[i]);
   free(plot.y);

   /* gif encoding */
   fp=fopen(ftmp, "wb");
   /* pixel_buffer is a gloabl variable which is the address for the image buffer */
   /* it is used by myGetGIFPixel to get the next values, so it have to be set to */
   /* buffer before calling to GIFEncode. The same for myrowlen.  bad coding method */
   pixel_buffer =plot.bit_map;
   mywidth = plot.xsize;
   greens = (int *)malloc(256*sizeof(int));
   reds = (int *)malloc(256*sizeof(int));
   blues = (int *)malloc(256*sizeof(int));
   for(i=0; i< 256; i++){
        reds[i]= i;
        blues[i]=128;
        greens[i]=255;
   }
   reds[0] = greens[0] = blues[0] = 0;
   reds[1] = greens[1] = blues[1] = 196;
   reds[2] = greens[2] = blues[2] = 255;
   reds[3] = greens[4] = blues[5] = 255;
   greens[3] = blues[3] = reds[4] = blues[4] =0;
   greens[5] = reds[5] = blues[6] = greens[7] = reds[8] = 0;
   reds[6] = greens[6] = 255;
   reds[7] = blues[7] = 255;
   greens[8] = blues[8] = 255;
  
   /* make legend at the end of the bit map. Not good, need to be redone */
   pt1 = plot.bit_map + (bysize - 30) * bxsize + G_LEFTMARGIN;
   yvtype = gxsize / (float)nvars + 0.5;
   for(i = 0; i<gxsize; i++){
       *pt1 = i/yvtype + 2;
       for(j = 1; j< 20; j++)
          *(pt1 + bxsize*j) = *pt1;
       pt1 ++;
  }
 
   GIFEncode(fp, plot.xsize, plot.ysize, 0, 0, 8, reds, greens, blues, GetGIFPixel_my);
   fclose(fp);
   free(plot.bit_map);
   return(1);
}
/*--------------------------------------------------------------------
 NAME
       tic_label
 DESCRIPTION
       prepare the tic and lable for plotting.
 RETURNS
       pointer to the label string pointers.
 AUTHOR
       Dr. Liping Di, Hughes STX.  
 COMMENTS
   
 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for any purpose without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.
---------------------------------------------------------------------*/

char ** tic_label (double vmin, double vmax, short npixels, unsigned short *start, unsigned short *step, 
                   unsigned short *ntics, unsigned short *mstart, unsigned short *nmtics)
{
  double a, b, dif, vstep, remain;
  long i;
  double fi, fstart, fstep;
  char **label;
  short i10=1, length, decimal=0; 
  char format[10];

  DBUG_PRINT(3, (LOGF, "in tic\n"));
  if( vmax == vmin ){ 
       *ntics = 1;
       *start = npixels/2;
       *step = npixels;
       fstart = vmax;
       fstep = 0.0001;  
       DBUG_PRINT(3, (LOGF, "fstart=%f, fstep=%f, ntic=%d\n", fstart, fstep, *ntics));
  }
  else {
       a = npixels/(vmax - vmin);
       b = -a * vmin;
       dif = vmax - vmin;

       vstep = dif/4;
       *ntics = 6; 
       fi = 1;
       remain = vstep; 
       if( vstep > 1){
           while (remain > 10){
                remain = remain / 10;
                fi = fi * 10; 
                decimal ++;
           }
       }
       else {
           while (remain < 1){
                 remain = remain * 10;
                 fi = fi/10;
                 decimal ++;
           }
      }
      fstep = ((long)(remain + 0.5)) * fi;
      fstart = ((long)(vmin/fstep)) * fstep;
      if(fstart < vmin) fstart += fstep;
      *start = (short)(a * fstart + b);
      *step = (short)(a * fstep+ 0.5);
  }    
  
  label = (char **)malloc (*ntics * sizeof (char **));
  for(i=0; i< *ntics; i++)
      label[i] = NULL;

  remain = fabs(vmax);
  if(remain < fabs(vmin)) remain = fabs(vmin);
  if(remain > 1){
      while( remain > 10){
          i10++;
          remain = remain/10;
       }
  }
  else {
      while(remain < 1 && remain != 0.0 ){
          i10--;
          remain = remain * 10;
      }
  } 
  if(fstep < 1){
     if(i10 >0)
        length = i10+decimal+2;
     else
        length = decimal + 2;
  }  
  else 
      length = i10;
    
  for (i = 0; i < *ntics; i++){
      label[i] = malloc(30);
      remain = fstart;
      if(*ntics == 1)
         sprintf(label[i], "%f", vmax);
      else if(length > 8){
         if(fstep < 1)
             sprintf(format, "%%.%dE", i10+decimal+1);
         else
             sprintf(format, "%%.%dE", i10-decimal);
         sprintf(label[i], format, fstart);
      }
      else {
          if(fstep < 1){
             sprintf(format,"%%.%df", decimal);
             sprintf(label[i], format, fstart);
          }
          else
             sprintf(label[i], "%d", (long)(fstart));
      } 
      if(*ntics == 1)break;
      DBUG_PRINT(3, (LOGF, "fstart=%f, fstep=%f, pixel=%d, %s %s\n", fstart, 
fstep, (long)(a*fstart + b), label[i], format));

      fstart = fstart + fstep;
      if( a * fstart + b > npixels)
           break;
  }
  if(*ntics != i ) *ntics = i + 1;
  DBUG_PRINT(3, (LOGF, "ntic=%d\n", *ntics));

  /* calculate the minor tics */
  if(vmax == vmin)
      *nmtics = 0;
  else {
      i = *start;
      i = i - (int)(*step/2.0 + 0.5);
      if ( i < 0) i = *start + (int)(*step/2.0 + 0.5);
      *mstart = (unsigned short)i;
      *nmtics = (npixels - *mstart) / *step + 1;
  }
  return label;
}


