/****************************************************************************
 * NCSA HDF                                                                 *
 * Software Development Group                                               *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * COPYING file.                                                            *
 *                                                                          *
 ****************************************************************************/
#ifdef RCSID
static char RcsId[] = "@(#)sdb_ri.c,v 1.18 1996/04/15 17:58:10 georgev Exp";
#endif

/* sdb_ri.c,v 1.18 1996/04/15 17:58:10 georgev Exp */

/*
   LIST Functions
   --------------
     add_rig      - Added raster to the raster List       
     pr_riglabel - Iterator fcn to print labels of raster 
     pr_rigdesc - Iterator fcn to print descriptions of raster 
     pr_rigimg - Iterator fcn to print raster dimensions and image 
     pr_rigdim - Iterator fcn to print raster dimensions and image 
     pr_rigpalette - Iterator fcn to print raster palette 
     rig_ann2html - Iterator fcn to print annotation of raster if any
     free_rig     - frees Raster element
     cleanup_rig  - Frees the raster Lists

   Regular Functions
   -----------------
     do_rigs   - Process the raster8 images in the file 
     dump_rigs - Print out the info for RIGSs in the file 

 */

/* Include our stuff */
#include "sdb.h"
#include "sdb_ri.h"
#include "sdb_ann.h"
#include "sdb_util.h"
#include "sdb_pal.h"
#include "glist.h"
#include "htmlstrings.h"
#include "html_util.h"

/*------------------------------------------------------------------------ 
 NAME
     add_rig - Added raster to the raster List       
 DESCRIPTION
     This adds the raster to the raster list
 RETURNS
     SUCCEED/FAIL
-------------------------------------------------------------------------*/
int
add_rig(Generic_list *rig_list,  /* Raster List */
        uint16 ref,              /* ref of raster */
        uint16 pal_ref,          /* ref of palette if any */
        intn have_pal,           /* flag */
        int32 w,                 /* width */
        int32 h,                 /* height */
        lvar_st *l_vars)
{
    trig_st *rig_elem = NULL;
    int     ret_value = SUCCEED;

    /* allocate space for raster element */
    if ((rig_elem = (trig_st *)HDmalloc(sizeof(trig_st))) == NULL)
      {
          gateway_err(l_vars->hfp,"add_rig: failed to allocate space for rig_elem \n",0);
          ret_value = FAIL; 
          goto done;
      }   

    /* fille in relavant info */
    rig_elem->ref      = ref;
    rig_elem->pal_ref  = pal_ref;
    rig_elem->have_pal = have_pal;
    rig_elem->width    = w;
    rig_elem->hieght   = h;

    /* add to list */
    add_to_beginning(*rig_list, rig_elem);
    DBUG_PRINT(3,(LOGF,"add_rig: adding raster,ref=%d \n",
                  rig_elem->ref));

  done:
    return ret_value;
} /* add_rig() */



/*------------------------------------------------------------------------ 
 NAME
     rig_ann2html - Iterator fcn to print annotation of raster if any
 DESCRIPTION
     Iterator fcn to print annotation of raster if any
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
rig_ann2html(void *dptr, /* raster element */
             void *args)
{
    tan_st *rig_ann = dptr;
    FILE *fout = args;

    if (rig_ann != NULL)
      {
          if (rig_ann->ann != NULL)
              prhtml_3str(fout,"<PRE>",rig_ann->ann,"</PRE>");
          else
              print_label(fout,"<PRE> -- </PRE>");

      }
} /* rig_ann2html() */

/*------------------------------------------------------------------------ 
 NAME
    pr_riglabel - Iterator fcn to print labels of raster 
 DESCRIPTION
     Iterator fcn to print raster labels to html tbl.
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
pr_riglabel(void *dptr, /* raster element */
             void *args)
{
    Generic_list *rig_data_label = dptr;
    lvar_st *l_vars = args;
    intn nlabels = 0;

    nlabels = num_of_objects(*rig_data_label);

    if (nlabels > 0)
       {
       perform_on_list(*rig_data_label,rig_ann2html,l_vars->hfp);
       }
    else
       fprintf(l_vars->hfp," -- ");
}


/*------------------------------------------------------------------------ 
 NAME
    pr_rigdesc - Iterator fcn to print descriptions of raster 
 DESCRIPTION
     Iterator fcn to print raster descriptions to html tbl.
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
pr_rigdesc(void *dptr, 
             void *args)
{
    Generic_list *rig_data_desc = dptr;
    lvar_st *l_vars = args;
    intn ndescs = 0;
    intn j;
    tan_st *rig_ann =NULL;

    ndescs = num_of_objects(*rig_data_desc);

    /* for descriptions dont print them out, list them */
                
    if (ndescs > 0)
      {
      print_label(l_vars->hfp, BGNUL);
      for (j =0; j < ndescs; j++)
        {
         rig_ann = next_in_list(*rig_data_desc);
         if (rig_ann != NULL)
         fprintf(l_vars->hfp,"<LI><A HREF=\"%s\"> desc%d </A></LI>",obj_href(rig_ann->ann_tag,rig_ann->ann_ref,rig_ann->index,l_vars),j); 
        }
      print_label(l_vars->hfp, ENDUL);
      }
    else
      fprintf(l_vars->hfp," -- ");
}

/*------------------------------------------------------------------------ 
 NAME
    pr_rigimg - Iterator fcn to print of raster image 
 DESCRIPTION
     Iterator fcn to print raster image to html tbl.
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
pr_rigimg(void *dptr, 
             void *args)
{
    trig_st *rig_elem = dptr;
    lvar_st *l_vars = args;

    fprintf(l_vars->hfp,"<A HREF=\"%s%s?%s!hdfref;tag=%d,ref=%d,s=%d\"><IMG SRC=\"%s%s?%s!hdfref;tag=%d,ref=%d,s=%d\"></A>",l_vars->h_env->script_name,l_vars->h_env->path_info,l_vars->f_name,(int32) DFTAG_RIG, rig_elem->ref, 0, l_vars->h_env->script_name,l_vars->h_env->path_info,l_vars->f_name,(int32)DFTAG_RIG, rig_elem->ref, 1);

}

/*------------------------------------------------------------------------ 
 NAME
    pr_rigdim - Iterator fcn to print of raster dimensions
 DESCRIPTION
     Iterator fcn to print raster dimensions to html tbl.
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
pr_rigdim(void *dptr, 
             void *args)
{
    trig_st *rig_elem = dptr;
    lvar_st *l_vars = args;

    fprintf(l_vars->hfp," %d by %d ",rig_elem->width, rig_elem->hieght);

}


/*------------------------------------------------------------------------ 
 NAME
    pr_rigpalette - Iterator fcn to print of raster palette 
 DESCRIPTION
     Iterator fcn to print raster palette to html tbl.
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
pr_rigpalette(void *dptr, 
             void *args)
{
    trig_st *rig_elem = dptr;
    lvar_st *l_vars = args;
    intn ndescs = 0;

    /* deal with palette if any */
    if (rig_elem->have_pal) 
    {     /* there is a pallette, print it */
       fprintf(l_vars->hfp, " <IMG SRC=\"%s%s?%s!hdfref;tag=%d,ref=%d,s=%d\"> ", l_vars->h_env->script_name,l_vars->h_env->path_info,l_vars->f_name, (int32) DFTAG_IP8, rig_elem->pal_ref,0);
                  }
                else
                    fprintf(l_vars->hfp," --  ");
}

/*------------------------------------------------------------------------ 

/*------------------------------------------------------------------------ 
 NAME
     free_rig - frees Raster element
 DESCRIPTION
     Frees Raster element
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
free_rig(void *dptr, /* raster element to free */
         void *args)
{
    trig_st *rig_elem = dptr;

    if (rig_elem != NULL)
        free(rig_elem);
} /* free_rig() */


/*------------------------------------------------------------------------ 
 NAME
     cleanup_rig - Frees the raster Lists
 DESCRIPTION
      Frees the raster Lists
 RETURNS
      Nothing
-------------------------------------------------------------------------*/
void
cleanup_rig(lvar_st *l_vars)
{
    ENTER(2,"cleanup_rig");
    /* Palette list */
    if (l_vars->rig_list != NULL)
      { /* free elements first */
          DBUG_PRINT(1,(LOGF,"cleanup_rig: destroy rig_list=%d\n", 
                        num_of_objects(*(l_vars->rig_list))));
          perform_on_list(*(l_vars->rig_list),free_rig,NULL);        
          /* destory list itself */
          destroy_list(l_vars->rig_list);

          free(l_vars->rig_list);
          l_vars->rig_list = NULL;
      }
    EXIT(2,"cleanup_rig");
} /* cleanup_rig() */


/*------------------------------------------------------------------------ 
 NAME
       do_rigs - Process the raster8 images in the file 
 DESCRIPTION
       Process the raster8 images in the file
 RETURNS
       SUCCEED/FAIL
-------------------------------------------------------------------------*/
int
do_rigs(char *fname,     /* file name */
        lvar_st *l_vars)
{
    int32  count;   /* counter for images ? */
    int32  i;       /* loop variable */
    int32  ref;     /* reference number for image */
    int32  status;  /* flag */
    int32  w;       /* width dimension of image */
    int32  h;       /* height dimesnion of image */
    intn   ip;       /* pallete flag */
    uint16 pal_ref;
    FILE   *h_fp  = NULL;
    int ret_value = SUCCEED;

    ENTER(2,"do_rigs");

    /* get number of raster images in file */
    if ((count = DFR8nimages(fname)) < 1)
      {
          ret_value = SUCCEED; 
          goto done;
      }

    /* initialize Palette lists */
    if(l_vars->rig_list == NULL)
      {
          /* allocate list to hold atributes */
          if ((l_vars->rig_list = HDmalloc(sizeof(Generic_list))) == NULL)
            {
                gateway_err(l_vars->hfp,"do_rigs: failed to allocate space for rigs list\n",0);
                ret_value = FAIL; 
                goto done;
            }   

          /* initialize list */
          initialize_list(l_vars->rig_list);
      }

    /* For each image in the file */
    for(i = 0; i < count; i++) 
      { /* Get width and height dimensions of image */
          if ((status = DFR8getdims(fname, &w, &h, &ip)) == FAIL)
            {
                gateway_err(h_fp,"do_rigs: failed to get dimension info on image",0);
                ret_value = FAIL;
                goto done;
            }

          /* get ref of palette if any*/
          if (ip)
              DFR8getpalref(&pal_ref);
          else
              pal_ref = 0;

          /* Save last reference number */
          if ((ref = DFR8lastref()) == FAIL)
            {
                gateway_err(h_fp,"do_rigs: failed to get last ref for image",0);
                ret_value = FAIL;
                goto done;
            }

          /* add rig to list */
          if (add_rig(l_vars->rig_list,ref,pal_ref,ip,w,h,l_vars) == FAIL)
            {
                gateway_err(h_fp,"do_rigs: failed to add rig to list \n",0);
                ret_value = FAIL;
                goto done;
            }

          /* now update palette list to reflect those that do not belong
           * to any image */
          if (l_vars->pal_list != NULL && ip)
              perform_on_list(*(l_vars->pal_list),set_lone_pal, &pal_ref);
      } /* end for each raster */

  done:
    /* failure */
    if (ret_value == FAIL)
      {
          cleanup_rig(l_vars);
      }
    EXIT(2,"do_rigs");
    return ret_value;
} /* do_rigs() */

/*------------------------------------------------------------------------ 
 NAME
       dump_rigs - Print out the info for RIGSs in the file 
 DESCRIPTION
       Print out the info for RIGSs in the file 
 RETURNS
       SUCCEED/FAIL
-------------------------------------------------------------------------*/
int
dump_rigs(char *fname,    /* file name */
          lvar_st *l_vars)
{
    int32 count;   /* counter for images ? */
    int32 i;     /* loop variable */
    intn  nlabels = 0;
    intn  ndescs  = 0;
    FILE         *h_fp     = NULL;
    trig_st      *rig_elem = NULL;
    Generic_list *rig_data_label = NULL;
    Generic_list *rig_data_desc  = NULL;
    tan_st       *rig_ann        = NULL;
    int ret_value = SUCCEED;
    Generic_list *rig_title  = NULL;

    /* Array of Descriptive Labels for Table Header Columns */
    char *hdr[MAXHDRCOLS] = { RI4, RI5, RI6, RI7, RI8};

    /* Number of Descriptive Labels for Table Header Columns */
    int numofhdrs =5;

    ENTER(2,"dump_rigs");
    /* Get number of raster 8 in file */
    if(l_vars->rig_list != NULL)
        count = num_of_objects(*(l_vars->rig_list));
    else
      {
          ret_value = SUCCEED;
          goto done;
      }

    /* how many rasters do we have */
    if (count < 1)
      {
          ret_value = SUCCEED; 
          goto done;
      }

    DBUG_PRINT(2,(LOGF,"dump_rigs: count=%d\n", count));
        h_fp = l_vars->hfp;

    /* Print Image header stuff in HTML  */
    prhtml_title(h_fp, RI1,count,RI2,RI9,RI10,RI3);

    DBUG_PRINT(1,(LOGF, "There are %d images in this file :\n", count));    
    

    /*  Print table headers */

     print_label(h_fp, HTMLBGNTBL);
     prhtml_tblhdr(h_fp,RI1,hdr,numofhdrs); 
	
    /* Prepare HTML table to hold images */

    /* For each image in the file */
    for(i = 0; i < count; i++) 
      { 
          /* get palette from list */
          rig_elem = next_in_list(*(l_vars->rig_list));

          /* Print out image info in HTML depending on user prefrences */
                /* get rig labels and descriptions if any */
                if ((rig_data_label = get_data_labels(fname, DFTAG_RIG, rig_elem->ref, h_fp,l_vars)) == NULL)
                  {
                      gateway_err(h_fp,"dump_rigs: error getting data labels",0);
                      ret_value = FAIL;
                      goto done;
                  }

                 if ((rig_data_desc = get_data_descs(fname, DFTAG_RIG, rig_elem->ref, h_fp,l_vars)) == NULL)
                  {
                      gateway_err(h_fp,"dump_rigs: error getting data descs",0);
                      ret_value = FAIL;
                      goto done;
                  }
                   print_label(h_fp, HTMLTBLROW2);
                  prhtml_tblentry(h_fp,(void *)(l_vars), (void *)rig_data_label, pr_riglabel);
                  prhtml_tblentry(h_fp,(void *)(l_vars), (void *)rig_data_desc, pr_rigdesc);
                  prhtml_tblentry(h_fp,(void *)(l_vars), (void *)(rig_elem),pr_rigdim);
                  prhtml_tblentry(h_fp,(void *)(l_vars), (void *)(rig_elem), pr_rigimg);
                  prhtml_tblentry(h_fp,(void *)(l_vars), (void *)(rig_elem), pr_rigpalette);
                  print_label(h_fp, HTMLTBLROWEND);
 

          /* cleanup of labels/descriptons */

          if (rig_data_label != NULL)
            {
                perform_on_list(*(rig_data_label),free_an,NULL);        
                destroy_list(rig_data_label);
                HDfree(rig_data_label);
                rig_data_label = NULL;
            }
          if (rig_data_desc != NULL)
            {
                perform_on_list(*(rig_data_desc),free_an,NULL);        
                destroy_list(rig_data_desc);
                HDfree(rig_data_desc);
                rig_data_desc = NULL;
            }


      } /* for loop for each image */

    print_label(h_fp, HTMLENDTBL);


  done:
    /* failure */
    if (ret_value == FAIL)
      {
      }

    /* cleanup of labels/descs */
    if (rig_data_label != NULL)
      {
          perform_on_list(*(rig_data_label),free_an,NULL);        
          destroy_list(rig_data_label);
          HDfree(rig_data_label);
          rig_data_label = NULL;
      }
    if (rig_data_desc != NULL)
      {
          perform_on_list(*(rig_data_desc),free_an,NULL);        
          destroy_list(rig_data_desc);
          HDfree(rig_data_desc);
          rig_data_desc = NULL;
      }
    EXIT(2,"dump_rigs");
    return ret_value;
} /* dump_rigs */
