/****************************************************************************
 * NCSA HDF                                                                 *
 * Software Development Group                                               *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * COPYING file.                                                            *
 *                                                                          *
 ****************************************************************************/
#ifdef RCSID
static char RcsId[] = "@(#)sdb_vd.c,v 1.23 1996/04/15 17:58:24 georgev Exp";
#endif

/* sdb_vd.c,v 1.23 1996/04/15 17:58:24 georgev Exp */

/*
   LIST Functions
   --------------
     add_vd     - Add vdata to Vdata List
     add_field  - Adds field to Field list of Vdata
     vd_cmp     - Iterator fcn to find the Vdata element given its ref
     free_field - frees field element
     free_vd    - frees Vdata element and its field list        
     cleanup_vd - Frees Vdata List      
     vd2htmlform - Iterator function to print out Vdata list as a html form      
     pr_vdfield2html - Iterator fcn to print vd field
     prhtml_field2checkbox - Iterator fcn to print vd field to html checkbox
     pr_vdfield2htmlselect - Iterator fcn to print vd field within a html tbl


   Regular Functions
   -----------------
     print_vdata   - Dump information about a vdata given its ref
     do_vd         - prccess a vdata
     do_lone_vds   - process all the Lone Vdatas in file and put them in the List  
     dump_lone_vds - dump info on the Lone Vdatas in the file

 */

/* Include our stuff */
#include "sdb.h"
#include "sdb_vd.h"
#include "show.h"
#include "sdb_util.h"
#include "glist.h"
#include "htmlstrings.h"
#include "html_util.h"

/*---------------------------------------------------------------------
 NAME
       add_vd - Add vdata to Vdata List
 DESCRIPTION
       Add vdata to Vdata List
 RETURNS
        SUCCEED/FAIL
-------------------------------------------------------------------------*/
int
add_vd(Generic_list *vd_list,   /* Vdata list */
       intn lone,               /* lone flag */
       int32 nelem,             /* number of record in VData */
       int32 nfields,           /* number of fields of Vdata */
       int32 interlaced,        /* interlaced flag */
       int32 size,              /* size */
       uint16 ref,              /* ref of VData */
       char *name,              /* name of Vdata */
       char *class,             /* class of Vdata */
       Generic_list *field_list, /* field List of Vdata */
       lvar_st *l_vars)
{
    tvd_st *vd_elem  = NULL;
    int    ret_value = SUCCEED;

    /* allocate space for Vdata element */
    if ((vd_elem = (tvd_st *)HDmalloc(sizeof(tvd_st))) == NULL)
      {
          gateway_err(l_vars->hfp,"add_vd: failed to allocate space for vd_elem\n",0);
          ret_value = FAIL; 
          goto done;
      }   

    /* allcate space for name of Vdata */
    if(name != NULL)
      {
          if ((vd_elem->name = (char *)HDmalloc(sizeof(char)*(HDstrlen(name)+1))) == NULL)
            {
                gateway_err(l_vars->hfp,"add_vd: failed to allocate space for vd_elem->name\n",0);
                ret_value = FAIL; 
                goto done;
            }
          HDstrcpy(vd_elem->name, name);
      }
    else
        vd_elem->name = NULL;

    /* allcate space for class of Vdata */
    if(class != NULL)
      {
          if ((vd_elem->class = (char *)HDmalloc(sizeof(char)*(HDstrlen(class)+1))) == NULL)
            {
                gateway_err(l_vars->hfp,"add_vd: failed to allocate space for vd_elem->class\n",0);
                ret_value = FAIL; 
                goto done;
            }
          HDstrcpy(vd_elem->class, class);
      }
    else
        vd_elem->class = NULL;

    /* fill in relevant info */
    vd_elem->lone       = lone;
    vd_elem->ref        = ref;
    vd_elem->nelem      = nelem;
    vd_elem->interlaced = interlaced;
    vd_elem->size       = size;
    vd_elem->nfields    = nfields;
    vd_elem->field_list = field_list;

    /* add to list */
    add_to_beginning(*vd_list, vd_elem);
    DBUG_PRINT(3,(LOGF,"add_vd: adding vdata,ref=%d,name=%s,class=%s,nelem=%d \n",
                  vd_elem->ref,vd_elem->name,vd_elem->class,vd_elem->nelem));

  done:
    /* failure */
    if (ret_value == FAIL)
      {
          if (vd_elem != NULL)
            {
                if(vd_elem->name != NULL)
                    HDfree(vd_elem->name);
                if(vd_elem->class != NULL)
                    HDfree(vd_elem->class);
                HDfree(vd_elem);
            }
      }
    return ret_value;
} /* add_vd() */

/*---------------------------------------------------------------------
 NAME
     add_field  - Adds field to Field list of Vdata
 DESCRIPTION
     Adds field to Field list of Vdata  
 RETURNS
     SUCCEED/FAIL
-------------------------------------------------------------------------*/
int
add_field(Generic_list *field_list, /* field List */
          char *field,              /* field to add to list */
          lvar_st *l_vars)
{
    char *f_elem   = NULL;
    int  ret_value = SUCCEED;

    /* allocate space for field element */
    if ((f_elem = (char *)HDmalloc(sizeof(char)*(HDstrlen(field)+1))) == NULL)
      {
          gateway_err(l_vars->hfp,"add_field: failed to allocate space for f_elem\n",0);
          ret_value = FAIL; 
          goto done;
      }   

    /* copy field */
    HDstrcpy(f_elem,field);

    /* add to list */
    add_to_beginning(*field_list, f_elem);
    DBUG_PRINT(3,(LOGF,"add_field: adding field=%s \n",
                  f_elem));

  done:
    return ret_value;
} /* add_field() */

/*---------------------------------------------------------------------
 NAME
     vd_cmp  - Iterator fcn to find the Vdata element given its ref
 DESCRIPTION
     Iterator fcn to find the Vdata element given its ref  
 RETURNS
     1-> SUCCESS, 0->FAIL
-------------------------------------------------------------------------*/
int
vd_cmp(void *dptr,  /* Vdata element */
       void *args)
{
    tvd_st *vd_elem = dptr;
    uint16 *ref     = args;

    if(vd_elem->ref == *ref)
        return 1;
    else
        return 0;
}/* vd_cmp() */

/*------------------------------------------------------------------------
 NAME
     vd2htmlform - Iterator fcn to print vd as a html form
 DESCRIPTION
     Iterator fcn to print vdata as a html select form
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void 
vd2htmlform(Generic_list list, FILE *fout, int numitems, char *itemlabel)
{
    int i=0;
    tvd_st *vd_elem;

    /* For each lone Vdata, list it  */
    reset_to_beginning(list);
    for(i = 0; i < numitems; i++) 
      {
          vd_elem = next_in_list(list);

                if (i==0)
                    fprintf(fout, "<OPTION SELECTED VALUE=\";tag=%d,ref=%d,s=%d\"> ",
                            DFTAG_VH, vd_elem->ref, 0);
                else	
                    fprintf(fout, "<OPTION VALUE=\";tag=%d,ref=%d,s=%d\"> ",
                            DFTAG_VH, vd_elem->ref, 0);
                fprintf(fout, "%s\n", vd_elem->name);
      } /* end for each lone Vdata */

   DBUG_PRINT(3,(LOGF,"vd2htmlform tag=%d,ref=%d,name=%s:\n",DFTAG_VH, vd_elem->ref, vd_elem->name));
 
 
} /* vd2htmlform() */

/*------------------------------------------------------------------------
 NAME
     pr_vdfield2html - Iterator fcn to print vd field
 DESCRIPTION
     vdfield2html - Iterator fcn to print vd field. May be used to
     print as a  html tbl entry
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void 
pr_vdfield2html(void *dptr,  /* Vdata element */
        void *args)
{
        char *field_name;
        int i;
        tvd_st *vd_elem = dptr;
        lvar_st *l_vars = args;

          if(vd_elem!= NULL)
            {
                reset_to_end(*(vd_elem->field_list));
                for (i = 1; i <= vd_elem->nfields; i++)
                  {
                      field_name = (char *)previous_in_list(*(vd_elem->field_list)); 
                       fprintf(l_vars->hfp, "%s ", field_name); 
                  }
            }
}

/*------------------------------------------------------------------------
 NAME
     prhtml_field2checkbox - Iterator fcn to print vd field in a html checkbox
 DESCRIPTION
     prhtml_field2checkbox - Iterator fcn to print vd field in a html checkbox
 RETURNS
     Nothing
-------------------------------------------------------------------------*/

void
prhtml_field2checkbox(tvd_st *vd_elem, FILE *fout, char *title)
{
    char *field_name;
    int i=1;
 
    if(vd_elem!= NULL)
            {
                fprintf(fout, "%s",title);
                reset_to_end(*(vd_elem->field_list));
                for (i = 1; i <= vd_elem->nfields; i++)
                  {
                      field_name = (char *)previous_in_list(*(vd_elem->field_list));
                      if( i == 1)
                         fprintf(fout, "<INPUT TYPE=\"checkbox\" NAME=\"VH_FIELD\" VALUE=\"%d\" checked>%s \n", i, field_name);
                      else
                         fprintf(fout, "<INPUT TYPE=\"checkbox\" NAME=\"VH_FIELD\" VALUE=\"%d\" >%s \n",
                              i,field_name);
                  }
	}
}

/*------------------------------------------------------------------------
 NAME
     pr_vdfield2htmlselect - Iterator fcn to print vd field as a select box within a html tbl
 DESCRIPTION
     vdfield2htmlselect - Iterator fcn to print vd field as a html select box. May be used to
     print as a  html tbl entry
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void 
pr_vdfield2htmlselect(void *dptr,  /* Vdata element */
        void *args)
{
        char *field_name;
        int i;
        tvd_st *vd_elem = dptr;
        lvar_st *l_vars = args;

          if(vd_elem!= NULL)
            {
                reset_to_end(*(vd_elem->field_list));
                for (i = 1; i <= vd_elem->nfields; i++)
                  {
                      field_name = (char *)previous_in_list(*(vd_elem->field_list)); 
                      if (i==0)
                         fprintf(l_vars->hfp, "<OPTION SELECTED VALUE=\";%s\"> ", field_name);
                      else	
                         fprintf(l_vars->hfp, "<OPTION VALUE=\";%s\"> ", field_name);
                      fprintf(l_vars->hfp, "%s\n", field_name);
                  }
            }
}


/*---------------------------------------------------------------------
 NAME
     free_field - frees field element
 DESCRIPTION
     frees field element  
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
free_field(void *dptr, /* field element */
           void *args)
{
    char *field = dptr;
    /* free structure itself */
    if(field != NULL)
        HDfree(field);
} /* free_field() */

/*---------------------------------------------------------------------
 NAME
     free_vd - frees Vdata element and its field list        
 DESCRIPTION
     frees Vdata element and its field list   
 RETURNS
     Nothing
-------------------------------------------------------------------------*/
void
free_vd(void *dptr, /* Vdata element to free */
        void *args)
{
    tvd_st *vd_elem = dptr;

    ENTER(2,"free_vd");
    if (vd_elem != NULL)
      {
          /* free elem list , can get recursive if non-NULL*/
          if (vd_elem->field_list != NULL)
            {
                DBUG_PRINT(1,(LOGF,"free_vd: destroy field_list=%d\n", 
                              num_of_objects(*(vd_elem->field_list))));
                perform_on_list(*(vd_elem->field_list),free_field,NULL);
                /* destroy element list */
                destroy_list(vd_elem->field_list);
                free(vd_elem->field_list);
            }

          /* free name and class */
          if (vd_elem->name != NULL)
              HDfree(vd_elem->name);
          if (vd_elem->class != NULL)
              HDfree(vd_elem->class);
          
          /* free structure itself */
          if(vd_elem != NULL)
              free(vd_elem);
      }
    EXIT(2,"free_vd");
} /* free_vd() */

/*---------------------------------------------------------------------
 NAME
     cleanup_vd - Frees Vdata List      
 DESCRIPTION
     Frees Vdata List  
 RETURNS
      Nothing
-------------------------------------------------------------------------*/
void
cleanup_vd(lvar_st *l_vars)
{
    ENTER(2,"cleanup_vd");
    /* now the Vdata list */
    if (l_vars->vd_list != NULL)
      { /* free elements first */
          DBUG_PRINT(1,(LOGF,"cleanup_vd: destroy vd_list=%d\n", 
                        num_of_objects(*(l_vars->vd_list))));
          perform_on_list(*(l_vars->vd_list),free_vd,NULL);        
          /* destory list itself */
          destroy_list(l_vars->vd_list);

          free(l_vars->vd_list);
          l_vars->vd_list = NULL;
      }
    EXIT(2,"cleanup_vd");
}/* cleanup_vd() */


/*---------------------------------------------------------------------
 NAME
       print_vdata - Dump information about a vdata given its ref
 DESCRIPTION
       Dump information about a vdata given its ref
 RETURNS
       SUCCEED/FAIL
-------------------------------------------------------------------------*/
int
print_vdata(int32 fid,       /* file handle */
            int32 ref,       /* reference number of Vdata */
            lvar_st *l_vars)
{
 int i;
    int32  intr;      /* type of interlace */
    int32  sz;        /* size of all Vdatas */
    int32  cnt;       /* number of Vdatas */
    int32  nfields;   /* number of fields in a Vdata */
    tvd_st *vd_elem    = NULL;
    char   *field_name = NULL;
    uint16 vdref;
    int32  count;
    FILE  *h_fp = l_vars->hfp;
    int ret_value = SUCCEED;
    int32 vdid, j, type;
    char title[120];
 
    /* Array of Descriptive Labels for Table Header Columns */
    char *hdr[MAXHDRCOLS] = { VD7,VD3,VD4,VD5,VD6,VD8 };
 
    /* Number of Descriptive Labels for Table Header Columns */
    int numofhdrs =6;
   
   /* Array of Radio Button Labels for Selecting Output Mode */
   /* first name in list is the name of "selected", default radio button */
    char *outputbuttons[MAXHDRCOLS] = { OUT10, OUT10, OUT11, OUT12, OUT13,OUT14 };
 
    /* Number of Output Buttons */
    int numradiobuttons =6;
 
 
 
    ENTER(2,"print_vdata");
    /* Get number of lone Vdatas i.e. not in any Vgroups */
    if(l_vars->vd_list != NULL)
      {  
          count = num_of_objects(*(l_vars->vd_list));
      }
    else
        count = 0;
 
    DBUG_PRINT(1,(LOGF, "print_vdata: There are %d Vdatas.\n",
                  count));
 
    if(count < 1)
      {
          ret_value = SUCCEED;
          goto done;
      }
 
    /* find Vdata from Vdata List using ref */
    vdref = (uint16)ref;
    reset_to_beginning(*(l_vars->vd_list));
    vd_elem = first_that(*(l_vars->vd_list),vd_cmp,&vdref);
 
    if (vd_elem != NULL)
      {
          cnt     = vd_elem->nelem;
          intr    = vd_elem->interlaced;
          nfields = vd_elem->nfields;
          sz      = (vd_elem->size * vd_elem->nelem);
          DBUG_PRINT(2,(LOGF, "print_vdata: nfields= %d", nfields));
    
          strcpy(title,VD14);
          strcat (title, vd_elem->name);
          prhtml_H2label(h_fp,title);
 
          /* print Vdata info in a table */
          print_label(h_fp, HTMLBGNTBL);
          prhtml_tblhdr(h_fp,"",hdr,numofhdrs);
 
          DBUG_PRINT(1,(LOGF, "print_vdata: Vdata %s", vd_elem->name));

          /* Print table containing :Vdata name ,class, */
          /* number of records, number of fields, size in bytes */
 
          print_label(h_fp, HTMLTBLROW1);
          prhtml_tblentry(h_fp, (void *)(l_vars),vd_elem->name, pr_tblstr);
          prhtml_tblentry(h_fp, (void *)(l_vars),(vd_elem->class[0] ? vd_elem->class : "--"), pr_tblstr);
          prhtml_tblentry(h_fp, (void *)(l_vars),&(vd_elem->nelem), pr_tblnum);
          prhtml_tblentry(h_fp, (void *)(l_vars),&(vd_elem->nfields), pr_tblnum);
          prhtml_tblentry(h_fp, (void *)(l_vars),&sz, pr_tblnum);
          prhtml_tblselectentry(h_fp, (void *)(l_vars), (void *)(vd_elem), pr_vdfield2htmlselect );
          print_label(h_fp, HTMLTBLROWEND);
          print_label(h_fp, HTMLENDTBL);


          fprintf(h_fp,"<FORM METHOD=\"POST\" ");
          fprintf(h_fp, "ACTION=\"%s%s\">\n",l_vars->h_env->script_name,
                  l_vars->h_env->path_info);
          fprintf(h_fp, "Toggle which fields you want to see when you select the Vdata");
          fprintf(h_fp, " or select one field as X axis variable for the xy plot : <p>");

          /* print out field list to select */
          if(vd_elem!= NULL)
            {
                reset_to_end(*(vd_elem->field_list));
                for (i = 1; i <= vd_elem->nfields; i++)
                  {
                      field_name = (char *)previous_in_list(*(vd_elem->field_list));
                      if( i == 1)
                         fprintf(h_fp, "<INPUT TYPE=\"checkbox\" NAME=\"VH_FIELD\" VALUE=\"%d\" checked>%s \n", i, field_name);
                      else
                         fprintf(h_fp, "<INPUT TYPE=\"checkbox\" NAME=\"VH_FIELD\" VALUE=\"%d\" >%s \n",
                              i,field_name);
                  }

                  /* inserted by lpd for select all */
                  fprintf(h_fp, "<P>or <INPUT TYPE=\"checkbox\" NAME=\"VH_FIELD\" VALUE=\"0\">");
                  fprintf(h_fp, " check here to select all fields\n");
                  fprintf(h_fp, "<INPUT TYPE=\"hidden\" NAME=\"NFIELDS\" VALUE=\"%d\">", vd_elem->nfields);
                  /* end the insert -- lpd */
            }
          fprintf(h_fp, "<P>");

          /* subsetting records */
          fprintf(h_fp, "Subsetting across Records: <P>\n");
          print_label(h_fp, BGNUL);
          fprintf(h_fp, "<LI>starting record :");
          fprintf(h_fp,"<INPUT NAME=\"%s\" VALUE=\"%s\"> \n",  "VH_START", "1");
          fprintf(h_fp, "<LI>skip factor:");       /* inserted by lpd */
          fprintf(h_fp,"<INPUT NAME=\"%s\" VALUE=\"%s\"> \n",  "VH_SKIP", "1");
          fprintf(h_fp, "<LI>ending record:");
          fprintf(h_fp, "<INPUT NAME=\"VH_END\" VALUE=%d> <P>\n", 
                  cnt);
          print_label(h_fp, ENDUL);
          fprintf(h_fp, "<P>");

	      /* add x-y plot option by lpd */
          fprintf(h_fp, "\n<b>The following selections are for X-Y plot only</b><p>");
		  fprintf(h_fp, "Select fields used as the Y axis values (numerical fields):<p>");

          /* y axis variables selections, only numerical variables are allowed */
          if(vd_elem!= NULL){
                vdid = VSattach(fid, ref, "r");
                if(vdid == FAIL) return FAIL;
                reset_to_end(*(vd_elem->field_list));
                j = 1;
                for (i = 1; i <= vd_elem->nfields; i++){
                    field_name = (char *)previous_in_list(*(vd_elem->field_list));
                    type = VFfieldtype(vdid, i-1);
                    if(type != DFNT_CHAR8 && type != DFNT_UCHAR8){
                        if( j == 1){
                            j = 2;
                            fprintf(h_fp, "<INPUT TYPE=\"checkbox\" NAME=\"VH_YFIELD\" VALUE=\"%d\" checked>%s \n", i, field_name);
                        }
                        else
                            fprintf(h_fp, "<INPUT TYPE=\"checkbox\" NAME=\"VH_YFIELD\" VALUE=\"%d\" >%s \n",
                                         i,field_name);
                    }
               }
				VSdetach(vdid);
          }

          fprintf(h_fp, "<p>If you select multiple variables for Y axis, please indicate how to fit the variables into Y by selecting the scaling method:<p>");
          fprintf(h_fp, "<pre><INPUT type=\"radio\" NAME=\"scaling\" VALUE=1 checked> relative  ");
          fprintf(h_fp, "<INPUT type=\"radio\" NAME=\"scaling\" VALUE=2> absolute\n<p>");
          fprintf(h_fp, "<p>Plot type:  <select NAME=Plot_style>");
          fprintf(h_fp, "<option selected>Dot<option>Solid line");
          fprintf(h_fp, "<option>Dot line_1<option>Dot line_2<option>Dot line_3");
          fprintf(h_fp, "<option>Dot line_4<option>Dot line_5<option>Dot line_6");
          fprintf(h_fp, "<option>Dot line_7</select>");
          fprintf(h_fp, "         Plot title:<INPUT NAME=Plot_title VALUE=>");
          fprintf(h_fp, "<p>X-axis name:<INPUT NAME=X_name VALUE=>");
          fprintf(h_fp, "   Y-axis name:<INPUT NAME=Y_name VALUE=>");
          fprintf(h_fp, "</pre>");     /* end the insert--lpd */
          prhtml_select_outputtype(h_fp, OUT5, outputbuttons, numradiobuttons);
          prhtml_submitbutton(h_fp, OUT1, OUT2, OUT3, DFTAG_VH, ref,0,l_vars->f_name, 3);
	  print_label(h_fp,"</FORM>");

      } /* if vd_elem */

  done:
    if(ret_value == FAIL)
      {
      }
    EXIT(2,"print_vdata");
    return ret_value;
} /* print_vdata */

/*----------------------------------------------------------------------------
 NAME
       do_vd - prccess a vdata
 DESCRIPTION  
       dump either info on the Vdata or the Vdata itself
 RETURNS
       SUCCEED/FAIL
-----------------------------------------------------------------------------*/
int
do_vd(char *fname,                 /* file name */
      int32 t,                     /* tag */
      int32 r,                     /* ref */
      lvar_st *l_vars)
{
    int32 i,j;
    int32 fid = FAIL;
    int32 vd_id = FAIL;
    char  sep[2];
    int   flds_indices[100];
    char   name[VSNAMELENMAX + 1];  /* Vdata name */
    char   class[VSNAMELENMAX + 1]; /* Vdata class */
    char   fields[(FIELDNAMELENMAX + 1) * VSFIELDMAX]; /* Vdata field names*/
    int32  nfields;
    int32  intr;      /* type of interlace */
    int32  sz;        /* size of Vdata */
    int32  cnt;       /* number of Vdatas */
    int x;
    uint16 vdref;
    Generic_list *field_list = NULL;
    char   *fptr = NULL;
    char   *nptr = NULL;
    FILE   *h_fp = l_vars->hfp;
    int ret_value = SUCCEED;

    ENTER(2,"do_vd");

    DBUG_PRINT(3,(LOGF,"do_vd: lets open file \n"));

    /* open file */
    if ((fid = Hopen(fname, DFACC_RDONLY, 0)) == FAIL)
      {
          gateway_err(l_vars->hfp,"do_vd: Problem opening HDF file",0);
          return FAIL;     /* return immediately --lpd-- */
      }

    /* start Vxx inteface */
    if ((Vstart(fid)) == FAIL)
      {
          gateway_err(l_vars->hfp,"do_vd: failed to start Vset interface\n",0); 
          Hclose(fid);
		  return FAIL;    /* return here in order to avoid calling Vend without Vstart -lpd */
      }

    /* attach to Vdata */
    if ((vd_id = VSattach(fid, r, "r")) == FAIL)
      {
          gateway_err(l_vars->hfp,"do_vd: failed to attach to Vdata\n",0);
          ret_value = FAIL; 
          goto done;
      }

    /* initialize Vdata lists */
    if(l_vars->vd_list == NULL)
      {
          /* allocate list to hold atributes */
          if ((l_vars->vd_list = HDmalloc(sizeof(Generic_list))) == NULL)
            {
                gateway_err(l_vars->hfp,"do_vd: failed to allocate space for SDS list\n",0);
                ret_value = FAIL; 
                goto done;
            }   

          /* initialize list */
          initialize_list(l_vars->vd_list);
      }

    /* get info on Vdata */
    if ((VSinquire(vd_id, &cnt, &intr, fields, &sz, name)) == FAIL)
      {
          gateway_err(l_vars->hfp,"do_vd: failed to get info on Vdata",0);
          ret_value = FAIL; 
          goto done;
      }

    /* Get class of Vdata */
    if ((VSgetclass(vd_id, class)) == FAIL)
      {
          Vdetach(vd_id); /* Dettach from Vdata */
          gateway_err(l_vars->hfp,"do_vd: failed to get class of Vdata",0);
          ret_value = FAIL; 
          goto done;
      }

    /* Get number of fields in Vdata */
    if ((nfields = VFnfields(vd_id)) == FAIL)
      {
          Vdetach(vd_id); /* Dettach from Vdata */
          gateway_err(l_vars->hfp,"do_vd: failed to number of fields in Vdata",0);
          ret_value = FAIL; 
          goto done;
      }

    /* Check if Vdata in list already */
    vdref = (uint16)r;
    if (first_that(*(l_vars->vd_list),vd_cmp,&vdref) == NULL)
      {
          /* prepare field list */
          if (nfields > 0)
            {
                /* allocate list to hold elements */
                if ((field_list = HDmalloc(sizeof(Generic_list))) == NULL)
                  {
                      Vdetach(vd_id); /* Dettach from Vdata */
                      gateway_err(l_vars->hfp,"do_vd: failed to allocate space for attribute list\n",0);
                      ret_value = FAIL; 
                      goto done;
                  }   

                /* initialize field list of this Vdata */
                initialize_list(field_list);

                /* add fields to field list */
                fptr = fields;
                for (j = 0; j < nfields; j++)
                  {
                      nptr = HDstrchr(fptr,',');
                      if (nptr != NULL)
                        {
                            *nptr = '\0';
                            nptr++;
                        }
                      if (add_field(field_list,fptr,l_vars) == FAIL)
                        {
                            Vdetach(vd_id); /* Dettach from Vdata */
                            gateway_err(h_fp,"do_vd: failed to add vdata to list \n",0);
                            ret_value = FAIL;
                            goto done;
                        }
                      fptr = nptr;
                  } /* end for nfields */
            }
          else
              field_list = NULL;

          /* add Vdata to Vdata list */
          if (add_vd(l_vars->vd_list,0,cnt,nfields,intr,sz,r,name,class,
                     field_list,l_vars) == FAIL)
            {
                Vdetach(vd_id); /* Dettach from Vdata */
                gateway_err(h_fp,"do_vd: failed to add vdata to list \n",0);
                ret_value = FAIL;
                goto done;
            }
      }/* if Vdata not in list already */

    /* detach from Vdata */
    VSdetach(vd_id);

    /* prepare Vdata HTML */

    DBUG_PRINT(3,(LOGF,"do_vd: get ready to dump Vdata\n"));

    /* Dump Vdata data 
     *  else dump vdata info */
    if (l_vars->display_mode == 3)
      { /* we are dumping the data itself */
          if ((vd_id = VSattach(fid, r, "r")) == FAIL)
            {
                gateway_err(l_vars->hfp,"do_vd: failed to attach to Vdata\n",0);
                ret_value = FAIL; 
                goto done;
            }

          for (x=0; x<100; x++)
              flds_indices[x] = -1;

          /* Debugging stuff */
          DBUG_PRINT(3,(LOGF,"do_vd: get ready to dump actual Vdata\n"));
          DBUG_PRINT(3,(LOGF,"do_vd: dumping fields ==>"));
          for (i = 0; i < l_vars->nfields; i++)
            {
                DBUG_PRINT(3,(LOGF,"%d,",l_vars->field_indices[i]));
            }
          DBUG_PRINT(3,(LOGF,"\n"));
          DBUG_PRINT(3,(LOGF,"do_vd: starting record=%d\n",l_vars->vh_start_rec));
          DBUG_PRINT(3,(LOGF,"do_vd: ending=%d\n",l_vars->vh_end_rec));
          DBUG_PRINT(3,(LOGF,"\n"));

          /* hmm.. */
          strcpy(sep, ";");

          /* Dump actual Vdata */
          if ((dumpvd(vd_id, FALSE, l_vars->hfp, sep, flds_indices, TRUE, l_vars)) == FAIL)
            {
                gateway_err(l_vars->hfp,"do_vd: to dump Vdata\n",0);
                ret_value = FAIL; 
                goto done;
            }

          /* detach from Vdata */
          VSdetach(vd_id);
      }
    else /* print info on Vdata in a Table */
        print_vdata(fid, r,l_vars);    


  done:
    /* failure */
    if (ret_value == FAIL)
      {
          cleanup_vd(l_vars);
      }

    if (fid != FAIL)
      {
          Vend(fid);
          Hclose(fid);
      }

    EXIT(2,"do_vd");
    return ret_value;
} /* do_vd() */

/*---------------------------------------------------------------------
 NAME
     do_lone_vds - process all the Lone Vdatas in file and put them in the List  
 DESCRIPTION
     process all the Lone Vdatas in file and put them in the List  
 RETURNS
     SUCCEED/FAIL
-------------------------------------------------------------------------*/
int
do_lone_vds(char *fname,      /* file name */
            lvar_st *l_vars)
{
    int32 count;        /* Number of lone Vdatas */
    int32 vd;           /* Vdata handle */
    int32 i,j;
    int32 status;      /* flag: function return ? */
    char   name[VSNAMELENMAX + 1];  /* Vdata name */
    char   class[VSNAMELENMAX + 1]; /* Vdata class */
    char   fields[(FIELDNAMELENMAX + 1) * VSFIELDMAX]; /* Vdata field names*/
    int32  intr;      /* type of interlace */
    int32  sz;        /* size of Vdata */
    int32  cnt;       /* number of Vdatas */
    int32  nfields;
    int32  fid = -1;
    int32 *ids = NULL;        /* Vdata */
    Generic_list *field_list = NULL;
    char   *fptr = NULL;
    char   *nptr = NULL;
    FILE   *h_fp;
    int    ret_value = SUCCEED;
    
    ENTER(2,"do_lone_vds");
    h_fp = l_vars->hfp;

    /* Open file */
    if ((fid = Hopen(fname, DFACC_RDONLY, 0)) == FAIL)
      {
          gateway_err(l_vars->hfp,"do_lone_vds: Problem opening HDF file",0);
          return NULL;
      }

    /* setup to start accessing Vgroups */
    if (Vstart(fid) == FAIL)
      {
          gateway_err(l_vars->hfp,"do_lone_vds: error with Vstart ",0);
          Hclose(fid);
          return NULL;
      }

    /* Get number of lone Vdatas i.e. not in any Vgroups */
    if ((count = VSlone(fid, NULL, 0)) < 1)
      {
          ret_value = SUCCEED;
          goto done;
      }

    /* initialize Vdata lists */
    if(l_vars->vd_list == NULL)
      {
          /* allocate list to hold atributes */
          if ((l_vars->vd_list = HDmalloc(sizeof(Generic_list))) == NULL)
            {
                gateway_err(l_vars->hfp,"do_lone_vds: failed to allocate space for SDS list\n",0);
                ret_value = FAIL; 
                goto done;
            }   

          /* initialize list */
          initialize_list(l_vars->vd_list);
      }
    
    /* allocate space for array of Vdata reference numbers  */
    if ((ids = (int32 *) HDgetspace(sizeof(int32) * count)) == NULL)
      {
          ret_value = FAIL; 
          goto done;
      }


    /* Get the array containg the lone Vdata reference numbers */
    if ((status = VSlone(fid, ids, count)) != count)
      {
          ret_value = FAIL; 
          goto done;
      }

    /* For each lone Vdata, add it to list */
    for(i = 0; i < count; i++) 
      {
          if ((vd = VSattach(fid, ids[i], "r")) == FAIL)
            {
                ret_value = FAIL; 
                goto done;
            }
          
          HDmemset(fields,'\0',(FIELDNAMELENMAX + 1) * VSFIELDMAX);
          /* Inquire about the Vdata */
          if ((VSinquire(vd, &cnt, &intr, fields, &sz, name)) == FAIL)
            {
                Vdetach(vd); /* Dettach from Vdata */
                gateway_err(l_vars->hfp,"do_lone_vds: failed to get info on Vdata",0);
                ret_value = FAIL; goto done;
            }

          /* Get class of Vdata */
          if ((VSgetclass(vd, class)) == FAIL)
            {
                Vdetach(vd); /* Dettach from Vdata */
                gateway_err(l_vars->hfp,"do_lone_vds: failed to get class of Vdata",0);
                ret_value = FAIL; goto done;
            }

          /* Get number of fields in Vdata */
          if ((nfields = VFnfields(vd)) == FAIL)
            {
                Vdetach(vd); /* Dettach from Vdata */
                gateway_err(l_vars->hfp,"do_lone_vda: failed to number of fields in Vdata",0);
                ret_value = FAIL; goto done;
            }

          /* prepare field list */
          if (nfields > 0)
            {
                /* allocate list to hold elements */
                if ((field_list = HDmalloc(sizeof(Generic_list))) == NULL)
                  {
                      Vdetach(vd); /* Dettach from Vdata */
                      gateway_err(l_vars->hfp,"do_lone_vdata: failed to allocate space for attribute list\n",0);
                      ret_value = FAIL; 
                      goto done;
                  }   

                /* initialize field list of this Vdata */
                initialize_list(field_list);

                /* add fields to field list */
                fptr = fields;
                for (j = 0; j < nfields; j++)
                  {
                      nptr = HDstrchr(fptr,',');
                      if (nptr != NULL)
                        {
                            *nptr = '\0';
                            nptr++;
                        }
                      if (add_field(field_list,fptr,l_vars) == FAIL)
                        {
                            Vdetach(vd); /* Dettach from Vdata */
                            gateway_err(h_fp,"do_lone_vds: failed to add vdata to list \n",0);
                            ret_value = FAIL;
                            goto done;
                        }
                      fptr = nptr;
                  } /* end for nfields */
            }
          else
              field_list = NULL;

          /* add Vdata to lone Vdata list */
          if (add_vd(l_vars->vd_list,1,cnt,nfields,intr,sz,ids[i],name,class,
                     field_list,l_vars) == FAIL)
            {
                Vdetach(vd); /* Dettach from Vdata */
                gateway_err(h_fp,"do_lone_vds: failed to add vdata to list \n",0);
                ret_value = FAIL;
                goto done;
            }

          /* detach from Vdata */
          VSdetach(vd);
      } /* for each lone Vdata */

  done:
    if (ret_value == FAIL)
      {
          cleanup_vd(l_vars);
      }

    /* free up allocated space */
    if (ids!= NULL)
        HDfreespace((void *)ids); /* Free Vdata reference number array */

    /* clean up */
    if (fid != FAIL)
      {
          Vend(fid);
          Hclose(fid);
      }
    EXIT(2,"do_lone_vds");
    return ret_value;
} /* do_lone_vds() */

/*----------------------------------------------------------------------------
 NAME
       dump_lone_vds - dump info on the Lone Vdatas in the file
 DESCRIPTION  
       dump info on the Lone Vdatas in the file
 RETURNS
       SUCCEED/FAIL
-----------------------------------------------------------------------------*/
int
dump_lone_vds(char *fname,     /* file name */
              lvar_st *l_vars)
{
    
    int32 count;        /* Number of lone Vdatas */
    tvd_st *vd_elem = NULL;
    FILE   *h_fp;
    int    ret_value = SUCCEED;
    
    ENTER(2,"dump_lone_vds");
    /* Get number of lone Vdatas i.e. not in any Vgroups */
    if(l_vars->vd_list != NULL)
      {
          count = num_of_objects(*(l_vars->vd_list));
      }
    else
        count = 0;

    DBUG_PRINT(1,(LOGF, "dump_lone_vds: There are %d Vdatas visible at this level of the file.\n", 
                  count));

    if(count < 1) 
      {
          ret_value = SUCCEED;
          goto done;
      }
    
        h_fp = l_vars->hfp;

    /* set up Vdata HTML stuff */
    prhtml_title(h_fp, VD10,count,VD11,VD12,VD13,NULL);

    DBUG_PRINT(1,(LOGF, "dump_lone_vds: There are %d Vdatas visible at this level of the file.\n", 
                  count));

      prhtml_form(h_fp, NULL, l_vars->h_env->script_name,l_vars->h_env->path_info, VD9, 5,VD2,VD1, l_vars->f_name,2, *(l_vars->vd_list), vd2htmlform,count, NULL);
 
  done:
    EXIT(2,"dump_lone_vds");
    return ret_value;
} /* dump_lone_vds */
